/**************************************************************************
 *
 *     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 *     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 *     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 *     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 *     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 *     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *     FOR A PARTICULAR PURPOSE.
 *
 *     (c) Copyright 2010 Xilinx, Inc.
 *     All rights reserved.
 *
 **************************************************************************/
/**************************************************************************
 * Filename:     menu.c
 *
 * Description:
 * Menu that runs the board test applications.
 *
 *
 * MODIFICATION HISTORY:
 *
 * Ver   Who  Date     Changes
 * ----- ---- -------- -------------------------------------------------------
 * 1.00  cmc  8-27-09  Initial Release
 * 2.00  jk   17-10-14 Evaluation of EdkDSP accelerators (Jiri Kadlec, UTIA)
 *
 ******************************************************************************/
#include "menu.h"
#include "board_test_app.h"
#include <ctype.h>
#include <xil_types.h>
#include "stdio.h"
#include "xparameters.h"
#include "xil_cache.h"

//#define SG_PLBIFACE_COSIM

#include "wal.h"
#include "wal_bce_jk.h"

#define SIMD  (8)

#ifdef SG_PLBIFACE_COSIM
#include "sg_plbiface.h"
WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, SG_PLBIFACE_ConfigTable, 0, SIMD, 0);
#else
#include "bce_fp12_1x8_0_axiw.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 0,
		SIMD, 0);
WAL_REGISTER_WORKER(worker2, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 1,
		SIMD, 0);
WAL_REGISTER_WORKER(worker3, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 2,
		SIMD, 0);
WAL_REGISTER_WORKER(worker4, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 3,
		SIMD, 0);
WAL_REGISTER_WORKER(worker5, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 4,
		SIMD, 0);
WAL_REGISTER_WORKER(worker6, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 5,
		SIMD, 0);
#endif

#include "fill_FA1101P0_program_store.h"
#include "fill_FA1101P1_program_store.h"

#include "fill_FP1101P0_program_store.h"
#include "fill_FP1101P1_program_store.h"

#if SG_PLBIFACE_COSIM
#define NN  (10)
#else
#define NN  (250)
#endif

#define ERR (0.0f)

float af[SIMD][NN], bf[SIMD][NN], zf[SIMD][NN];
float afr[SIMD][NN], bfr[SIMD][NN], zfr[SIMD][NN];

void draw_pb2mb(wal_worker_t *wrk) {
	uint32_t status_led, status_screen;

	wal_pb2mb(wrk, &status_screen);
	while ((status_screen & 0x00000007) != 0) {
		wal_pb2mb(wrk, &status_led);
		while ((status_led & 0x02000000) == 0) {
			wal_pb2mb(wrk, &status_led);
		}
		wal_pb2mb(wrk, &status_screen);
	}
	wal_pb2mb(wrk, &status_led);
	while ((status_led & 0x02000000) == 0) {
		wal_pb2mb(wrk, &status_led);
	}
}

float vprod_mb(unsigned int nn, float * afmb, float * bfmb) {
	unsigned int i, j;
	float zs[4];

	for (i = 0; i < 4; i++) {
		zs[i] = 0;
	}

	j = 0;
	for (i = 0; i < nn; i++) {
		zs[j] += afmb[i] * bfmb[i];
		if (j < 3)
			j = j + 1;
		else
			j = 0;
	}
	zs[0] += zs[1];
	zs[2] += zs[3];
	zs[0] += zs[2];

	return ((float) zs[0]); //vector product
}

void vmac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] += afmb[i] * bfmb[i];
	}
}

void vmsubac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] -= afmb[i] * bfmb[i];
	}
}

void bce_fp12_1x8_0_VZ2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2A '%s' .......", wrk->name);

// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2A;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				zf[j][i] = (float) i + (float) j + (float) 0.123456;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (afr[j][i] != zf[j][i]) {
					xil_printf("\r\n VZ2A Error: afr[%d][%d] = %d", j, i,
							(int) afr[j][i]);
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2A not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VB2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VB2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VB2A '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VB2A;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				bf[j][i] = (float) i + (float) j + (float) 0.234567;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}
		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (afr[j][i] != bf[j][i]) {
					xil_printf("\r\n VB2A Error: afr[%d][%d] = %d", j, i,
							(int) afr[j][i]);
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VB2A not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VZ2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2B '%s' .......", wrk->name);

// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2B;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				zf[j][i] = (float) i + (float) j + (float) 0.123456;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (bfr[j][i] != zf[j][i]) {
					xil_printf("\r\n VZ2B Error: bfr[%d][%d] = %d", j, i,
							(int) bfr[j][i]);
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2B not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VA2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VA2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VA2B '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VA2B;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.234567;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (ah * 0x100), &af[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (ah * 0x100), &bfr[j][0],
					nn);
		}
		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (bfr[j][i] != af[j][i]) {
					xil_printf("\r\n VA2B Error: bfr[%d][%d] = %d", j, i,
							(int) bfr[j][i]);
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VA2B not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VADD(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] + bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VADD_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_BZ2A '%s' ..", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] + zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_BZ2A not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VADD_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_AZ2B '%s' ..", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] + zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_AZ2B not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VSUB(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] - bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VSUB_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_BZ2A '%s' ..", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] - zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_BZ2A not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VSUB_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_AZ2B '%s' ..", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] - zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_AZ2B not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VMULT(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT '%s' ......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] * bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VMULT_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_BZ2A '%s' .", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] * zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_BZ2A not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VMULT_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_AZ2B '%s' .", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] * zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_AZ2B not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VPROD(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD '%s' ......", wrk->name);
		//definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0], 1);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			r = vprod_mb(nn, &af[j][0], &bf[j][0]);
			err = zfr[j][0] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VPROD Error: zfr[%d][0] = %d   r = %d",
					(int) zfr[j][0], (int) r);
			k++;
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VMAC(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMAC '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMAC;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
				zf[j][i] = (float) 0.0;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			vmac_mb(nn, &af[j][0], &bf[j][0], &zf[j][0]);
			for (i = 0; i < nn; i++) {
				err = zf[j][i] - zfr[j][i];
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMAC Error: zf[%d][%d] = %d   zf1r[%d] = %d", j, i,
						(int) zf[j][i], i, (int) zfr[j][i]);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMAC not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VMSUBAC(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMSUBAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMSUBAC '%s' ....", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMSUBAC;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
				zf[j][i] = (float) 0.0;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			vmsubac_mb(nn, &af[j][0], &bf[j][0], &zf[j][0]);
			for (i = 0; i < nn; i++) {
				err = zf[j][i] - zfr[j][i];
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMSUBAC Error: zf[%d][%d] = %d   zf1r[%d] = %d", j, i,
						(int) zf[j][i], i, (int) zfr[j][i]);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMSUBAC not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VPROD_S8(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD_S8) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD_S8 '%s' ...", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD_S8;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 2.76543;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0], 1);
		}

		k = 0;
		r = ((vprod_mb(nn, &af[0][0], &bf[0][0])
				+ vprod_mb(nn, &af[1][0], &bf[1][0]))
				+ (vprod_mb(nn, &af[2][0], &bf[2][0])
						+ vprod_mb(nn, &af[3][0], &bf[3][0])))
				+ ((vprod_mb(nn, &af[4][0], &bf[4][0])
						+ vprod_mb(nn, &af[5][0], &bf[5][0]))
						+ (vprod_mb(nn, &af[6][0], &bf[6][0])
								+ vprod_mb(nn, &af[7][0], &bf[7][0])));

		for (j = 0; j < SIMD; j++) {
			err = zfr[j][0] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VPROD_S8 Error: zfr[%d][0] = %d   r = %d", j,
					(int) zfr[j][0], (int) r);
			k++;
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD_S8 not supported");
	}

	//====================================================
}

void bce_fp12_1x8_0_VDIV(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VDIV) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VDIV '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VDIV;

		for (i = 0; i < nn; i++) {
			af[0][i] = (float) i + (float) 0.123456;
			bf[0][i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[0][0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[0][0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[0][i] / bf[0][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VDIV Error: zfr[%d][%d] = %d   r = %d", j, i,
						(unsigned int) zfr[j][i], (unsigned int) r);
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VDIV not supported");
	}
	//====================================================
}

void update_firmware(FILE *fr, unsigned int position,
		unsigned int *fill_FP1101P0_program_store) {
	unsigned int j, k;

	for (k = 0; k < 3; k++) {
		if (feof(fr) == 0) {
			fscanf(fr, "%d", &j);
			fill_FP1101P0_program_store[k] = j;
		}
	}
	for (k = position; k < 1024; k++) {
		if (feof(fr) == 0) {
			fscanf(fr, "%d", &j);
			fill_FP1101P0_program_store[k] = j;
		} else {
			fill_FP1101P0_program_store[k] = 0;
		}
	}
}

int hello_eval_op() {

	unsigned int i;
	unsigned int ah, bh, zh;
	unsigned int capabilities1, license1;
	unsigned int capabilities2, license2;
	unsigned int capabilities3, license3;
	unsigned int capabilities4, license4;
	unsigned int capabilities5, license5;
	unsigned int capabilities6, license6;

	if (wal_init_worker(worker1)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	if (wal_init_worker(worker2)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	if (wal_init_worker(worker3)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	if (wal_init_worker(worker4)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	if (wal_init_worker(worker5)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	if (wal_init_worker(worker6)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}

	xil_printf("-- Entering main() --\r\n");
	xil_printf("\r\n Tests of vector operations.");

	///////////////////////////////////////////////
	// Upgrade firmware
	for (i = 0; i < 3; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}
	for (i = EDKDSP_BOOT; i < WAL_FW_FULL_SIZE; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities1 = %x", capabilities1);

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker2, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker2, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker2, WAL_PBID_P0, &capabilities2);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities2 = %x", capabilities2);

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker3, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker3, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker3, WAL_PBID_P0, &capabilities3);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities3 = %x", capabilities3);

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker4, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker4, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker4, WAL_PBID_P0, &capabilities4);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities4 = %x", capabilities4);

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker5, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker5, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker5, WAL_PBID_P0, &capabilities5);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities5 = %x", capabilities5);

	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker6, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker6, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker6, WAL_PBID_P0, &capabilities6);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities6 = %x", capabilities6);

	for (ah = 0; ah < 4; ah++) {
		for (bh = 0; bh < 4; bh++) {
			for (zh = 0; zh < 4; zh++) {
				// Test of the HW evaluation license
				wal_get_license(worker1, WAL_PBID_P0, &license1);
				if (license1) {
					xil_printf("\r\n\r\n ah=%d  bh=%d  zh=%d", ah, bh, zh);
					bce_fp12_1x8_0_VZ2A(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VB2A(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VZ2B(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VA2B(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VADD(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VADD_BZ2A(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VADD_AZ2B(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VSUB(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VSUB_BZ2A(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VSUB_AZ2B(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VMULT(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VMULT_BZ2A(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VMULT_AZ2B(worker1, capabilities1, ah, bh,
							zh);
					bce_fp12_1x8_0_VPROD(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VMAC(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VMSUBAC(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VPROD_S8(worker1, capabilities1, ah, bh, zh);
					bce_fp12_1x8_0_VDIV(worker1, capabilities1, ah, bh, zh);
				} else {
					xil_printf(
							"\r\n\r\n Evaluation license will expire soon. Please contact:");
					xil_printf(
							"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
					xil_printf("\r\n Pod vodarenskou vezi 4");
					xil_printf("\r\n 18208, Prague 8, Czech republic");
					xil_printf("\r\n http://zs.utia.cas.cz \r\n");
				}
			}
		}
	}
	xil_printf("\r\n");
	return (0);
}

/*
 * Macros to enable/disable caches.
 */
#ifndef ENABLE_ICACHE
#define ENABLE_ICACHE()		Xil_ICacheEnable()
#endif
#ifndef	ENABLE_DCACHE
#define ENABLE_DCACHE()		Xil_DCacheEnable()
#endif
#ifndef	DISABLE_ICACHE
#define DISABLE_ICACHE()	Xil_ICacheDisable()
#endif
#ifndef DISABLE_DCACHE
#define DISABLE_DCACHE()	Xil_DCacheDisable()
#endif

extern int hello_rotary(void);
extern int hello_bram(void);
extern int main_sysmonIntr(void);
extern int hello_mem_complete(void);

int main() {
	int choice, exit_flag;

	print("\n\r********************************************************");
	print("\n\r********************************************************");
	print("\n\r**      Xilinx Artix-7 FPGA AC701 Evaluation Kit      **");
	print("\n\r********************************************************");
	print("\n\r********************************************************\r\n");

	exit_flag = 0;
	while (exit_flag != 1) {
		ENABLE_ICACHE();
		ENABLE_DCACHE();
		//do {
		print("Choose Feature to Test:\r\n");
		print("1: UART Test\r\n");
		print("2: LED Test\r\n");
		print("3: IIC Test\r\n");
		print("4: TIMER Test\r\n");
		print("5: ROTARY Test\r\n");
		print("6: SWITCH Test\r\n");
		print("7: LCD Test\r\n");
		print("8: DDR3 External Memory Test\r\n");
		print("9: BRAM Internal Memory Test\r\n");
		print("A: ETHERNET Loopback Test\r\n");
		print("B: BUTTON Test\r\n");
		print("C: EdkDSP Eval Op\r\n");
		print("0: Exit\r\n");
		choice = inbyte();
		if (isalpha(choice)) {
			choice = toupper(choice);
		}
		//} while (!isdigit(choice));
		xil_printf("%c\r\n", choice);

		switch (choice) {
		case '0':
			exit_flag = 1;
			break;
		case '1':
			hello_uart(UART_BAUDRATE);
			break;
		case '2':
			hello_led();
			break;
		case '3':
			hello_iic();
			break;
		case '4':
			hello_timer();
			break;
		case '5':
			hello_rotary();
			break;
		case '6':
			hello_switch();
			break;
		case '7':
			lcd_simple();
			break;
		case '8':
			hello_mem();
			break;
		case '9':
			hello_bram();
			break;
		case 'A':
			hello_temac();
			break;
		case 'B':
			hello_button();
			break;
		case 'C':
			hello_eval_op();
			break;
		default:
			break;
		}
		if (exit_flag != 1) {
			print("Press any key to return to main menu\r\n");
			inbyte();
		}
	}
	print("Good-bye!\r\n");

	return 0;
}
